//==============================================
//	FT_MPSSE FTDI FT2232C MPSSE JTAG module		
//		2005/03/01		Ver 0.0					
//												
//	Compiled Visual C++ Ver5.0 with D2XX DLL ver.1.11
//												
//	Windows98SE    with D2XX Driver ver.1.11	
//	Windows2000Pro with D2XX Driver ver.1.11	
//												
//		Programed By	Y.Sakamoto				
//		Copyright (C)2005 Y.Sakamoto			
//		http://www.neko.ne.jp/~freewing/
//==============================================

#include <stdio.h>
#include <windows.h>

#include "Ftd2xx.h"
#include "FT_Mpsse.h"


//==============================================
//==============================================
FT_HANDLE	ftHandle	= NULL;


//==============================================
//	FTDI Subroutine
//==============================================
BOOL	FTDI_OpenDevice(void)
{
	FT_STATUS	ftStatus;
	DWORD		dwDeviceCount;

	ftHandle	= NULL;

	//	List FTDI device
	ftStatus	= FT_ListDevices(&dwDeviceCount, NULL, FT_LIST_NUMBER_ONLY);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);
	if (dwDeviceCount < 1)				return (FALSE);

	//	Open
	ftStatus	= FT_Open(0, &ftHandle);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_CloseDevice(void)
{
	FT_STATUS	ftStatus;

	if (!ftHandle)						return (FALSE);

	//	Close FTDI device
	ftStatus	= FT_Close(ftHandle);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	return (TRUE);
}


//==============================================
BOOL    FTDI_SetMpsse(void)
{
	FT_STATUS	ftStatus;
	BYTE		abyBuf[10];
	DWORD		dwBytesWritten;

	if (!ftHandle)						return (FALSE);

	//	Initialize FT2232C to MPSSE
	//	Purge Buffer
	ftStatus	= FT_Purge(ftHandle, FT_PURGE_RX);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	//	Set Latency Timer 3ms
	ftStatus	= FT_SetLatencyTimer(ftHandle, 3);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	//	Reset controller
	ftStatus	= FT_SetBitMode(ftHandle, 0x00, 0x00);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	//	Enable JTAG(MPSSE) controller
	ftStatus	= FT_SetBitMode(ftHandle, 0x00, 0x02);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	//	Purge Buffer
	ftStatus	= FT_Purge(ftHandle, FT_PURGE_RX);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	//	Wait for all the USB stuff to complete
	Sleep(20);

//	//	Set Timeouts (Read)
//	ftStatus	= FT_SetTimeouts(ftHandle, 1000, 1000);
//	if (!FT_SUCCESS(ftStatus))			return (FALSE);

//	//	Set USB Parameters
//	ftStatus	= FT_SetUSBParameters(ftHandle, 4096, 0);
//	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	//
	abyBuf[0]	= 0x80;	//  Set I/O Low Bits
	abyBuf[1]	= 0x00;	//  Port Data
	abyBuf[2]	= 0x0b;	//  Direction

	abyBuf[3]	= 0x82;	//  Set I/O High Bits
	abyBuf[4]	= 0x00;	//  Port Data
	abyBuf[5]	= 0x00;	//  Direction

	abyBuf[6]	= 0x86;						// Set Clk Divisor
	abyBuf[7]	= (BYTE)(0x00 & 0xFF);		// L,(0x0000=6MHz)
	abyBuf[8]	= (BYTE)((0x00>>8) & 0xFF);	// H

	abyBuf[9]	= 0x85;	//  Turn Off Loop Back

	//	Write
	ftStatus	= FT_Write(ftHandle, abyBuf, 10, &dwBytesWritten);
	if (!FT_SUCCESS(ftStatus))		return (FALSE);
	if (dwBytesWritten != 10)		return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_PurgeBuffer(void)
{
	FT_STATUS	ftStatus;

	if (!ftHandle)						return (FALSE);

	//	Purge Buffer
	ftStatus	= FT_Purge(ftHandle, FT_PURGE_RX);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_TransferBuffer(BYTE* pbyTxBuffer, DWORD dwTxSize)
{
	FT_STATUS	ftStatus;
	DWORD	dwBytesWritten;

	if (!ftHandle)						return (FALSE);

	if (!pbyTxBuffer)					return (FALSE);
	if (!dwTxSize)						return (FALSE);

	//	Write
	ftStatus	= FT_Write(ftHandle, pbyTxBuffer, dwTxSize, &dwBytesWritten);
	if (!FT_SUCCESS(ftStatus))	{	return (FALSE);	}
	if (dwBytesWritten != dwTxSize)		return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_WaitQueue(DWORD dwRxSize, DWORD dwTimeOutMs)
{
	FT_STATUS	ftStatus;
	DWORD		dwRxBytes;

	if (!ftHandle)						return (FALSE);

	do
	{
		//	Queue
		ftStatus	= FT_GetQueueStatus(ftHandle, &dwRxBytes);
		if (!FT_SUCCESS(ftStatus))	{	return (FALSE);	}
		if (dwRxBytes >= dwRxSize)		return (TRUE);

		Sleep(1);
	} while (dwTimeOutMs--);

	return (FALSE);
}


//==============================================
BOOL	FTDI_ReceiveBuffer(BYTE* pbyRxBuffer, DWORD dwRxSize)
{
	FT_STATUS	ftStatus;
	DWORD		dwBytesReturned;

	if (!ftHandle)						return (FALSE);

	if (!pbyRxBuffer)					return (FALSE);
	if (!dwRxSize)						return (FALSE);

	//	Read
	ftStatus	= FT_Read(ftHandle, pbyRxBuffer, dwRxSize, &dwBytesReturned);
	if (!FT_SUCCESS(ftStatus))	{	return (FALSE);	}
	if (dwBytesReturned != dwRxSize)	{	return (FALSE);	}

	return (TRUE);
}


//==============================================
BOOL	FTDI_WaitReceiveBuffer(BYTE* pbyRxBuffer, DWORD dwRxSize, DWORD dwTimeOutMs)
{
	if (!pbyRxBuffer)					return (FALSE);
	if (!dwRxSize)						return (FALSE);

	//	Wait
	if (FTDI_WaitQueue(dwRxSize, dwTimeOutMs) != TRUE)	return (FALSE);

	//	Read
	return (FTDI_ReceiveBuffer(pbyRxBuffer, dwRxSize));
}


//==============================================
BOOL	FTDI_SetClkDivisor(WORD wDivisor)
{
	BYTE	abyBuf[3];

	if (!ftHandle)						return (FALSE);

	//	Set Clk Divisor
	abyBuf[0]	= 0x86;							// Set Clk Divisor
	abyBuf[1]	= (BYTE)(wDivisor & 0xFF);		// L,(0x0000=6MHz)
	abyBuf[2]	= (BYTE)((wDivisor>>8) & 0xFF);	// H

	//	Transfer
	return (FTDI_TransferBuffer(abyBuf, 3));
}


//==============================================
//	2-255ms(16ms)
BOOL	FTDI_SetLatencyTimer(UCHAR ucTimer)
{
	FT_STATUS	ftStatus;

	if (!ftHandle)						return (FALSE);

	//	Set Latency Timer
	ftStatus	= FT_SetLatencyTimer(ftHandle, ucTimer);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_SetTimeouts(ULONG ulReadTimeout, ULONG ulWriteTimeout)
{
	FT_STATUS	ftStatus;

	if (!ftHandle)						return (FALSE);

	//	Set Timeouts
	ftStatus	= FT_SetTimeouts(ftHandle, ulReadTimeout, ulWriteTimeout);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_SetUSBParameters(ULONG ulInTransferSize, ULONG ulOutTransferSize)
{
	FT_STATUS	ftStatus;

	if (!ftHandle)						return (FALSE);

	//	Set USB Parameters
	ftStatus	= FT_SetUSBParameters(ftHandle, ulInTransferSize, ulOutTransferSize);
	if (!FT_SUCCESS(ftStatus))			return (FALSE);

	return (TRUE);
}


//==============================================
BOOL	FTDI_SetPortEx(BYTE byPortType, BYTE byPortData, BYTE byPortDir)
{
	BYTE	abyBuf[3];

	//	Set I/O Port
	if (byPortType == 0)	abyBuf[0]	= 0x80;	//	Set I/O Low Bits
	else					abyBuf[0]	= 0x82;	//	Set I/O High Bits
	abyBuf[1]	= byPortData;	//	Port Data
	abyBuf[2]	= byPortDir;	//	Direction

	//	Transfer
	return (FTDI_TransferBuffer(abyBuf, 3));
}


//==============================================
BOOL	FTDI_SetPort(BYTE byPortType, BYTE byPortData, BYTE byPortDir)
{

	//	Set I/O Port
	if (byPortType == 0)
	{
		//	Direction Guard [x,x,x,x,TMS,TDO,TDI,TCK]	=[xxxx_OIOO]
		byPortDir	|= 0x0B;
		byPortDir	&= ~0x04;
	}
	else
	{
		//	Direction Guard
		byPortDir	|= 0x00;
		byPortDir	&= ~0x00;
	}

	//	Transfer
	return (FTDI_SetPortEx(byPortType, byPortData, byPortDir));
}


//==============================================
BOOL	FTDI_GetPort(BYTE byPortType, BYTE* pbyPortData)
{
	BOOL	bFlag;
	BYTE	abyBuf[2];

	if (pbyPortData)		*(pbyPortData)	= 0;

	//	Get I/O Port
	if (byPortType == 0)	abyBuf[0]	= 0x81;	//	Get I/O Low Bits
	else					abyBuf[0]	= 0x83;	//	Get I/O High Bits
	abyBuf[1]	= 0x87;							//	Send Immediate

	//	Transfer
	bFlag	= FTDI_TransferBuffer(abyBuf, 2);
	if (!bFlag)				return (FALSE);

	//	Receive
	bFlag	= FTDI_WaitReceiveBuffer(abyBuf, 1, 1000);
	if (!bFlag)				return (FALSE);

	if (pbyPortData)		*(pbyPortData)	= abyBuf[0];

	return (TRUE);
}


//==============================================
BOOL	FTDI_SetParameter(UCHAR ucTimer, ULONG ulTimeout, ULONG ulInTransferSize, WORD wDivisor, BYTE byPortData)
{
	BOOL	bFlag;

	Sleep(20);	// wait for all the USB stuff to complete

	//	Purge Buffer
	bFlag	= FTDI_PurgeBuffer();
	if (!bFlag)		return (FALSE);

	Sleep(20);	// wait for all the USB stuff to complete

	//	Set Latency Timer
	bFlag	= FTDI_SetLatencyTimer(ucTimer);
	if (!bFlag)		return (FALSE);

	//	Set Timeouts (Read)
	bFlag	= FTDI_SetTimeouts(ulTimeout, 0);
	if (!bFlag)		return (FALSE);

	//	Set USB Parameters
	bFlag	= FTDI_SetUSBParameters(ulInTransferSize, 0);
	if (!bFlag)		return (FALSE);

	//	Set Clk Divisor
	bFlag	= FTDI_SetClkDivisor(wDivisor);
	if (!bFlag)		return (FALSE);

	Sleep(20);	// wait for all the USB stuff to complete

	//	Set I/O Port
	bFlag	= FTDI_SetPort(0, byPortData, 0x0B);
	if (!bFlag)		return (FALSE);

	//	Set I/O Port
	bFlag	= FTDI_SetPort(1, 0x08, 0x0C);
	if (!bFlag)		return (FALSE);

	//	Purge Buffer
	bFlag	= FTDI_PurgeBuffer();
	if (!bFlag)		return (FALSE);

	return (TRUE);
}


