//==============================================
//	FT_TEST MPSSE TEST PROGRAM					
//		2005/03/01		Ver 0.0					
//												
//	Compiled Visual C++ Ver5.0 with D2XX DLL ver.1.11
//												
//	Windows98SE    with D2XX Driver ver.1.11	
//	Windows2000Pro with D2XX Driver ver.1.11	
//												
//		Programed By	Y.Sakamoto				
//		Copyright (C)2005 Y.Sakamoto			
//		http://www.neko.ne.jp/~freewing/
//==============================================

#include <stdio.h>
#include <time.h>
#include <windows.h>

#include "FT_Mpsse.h"


//==============================================
#define	BUFFER_MAX	(0x10010)

#define	STR_OKNG(f)	((f)? "OK": "NG")

//==============================================
BOOL	FtTest(int nParam);
BOOL	TdiBench(int nParam);
BOOL	TdiBenchSub(int nLoopCnt, DWORD dwLength, BYTE byTxData, BOOL bTxEna, BOOL bRxEna, BOOL bSendImmEna);
BOOL	TmsSend(int nLoopCnt);


//==============================================
void	main(int argc, char* argv[])
{
	BOOL	bFlag;

	//==============================================
	printf("============================================\r\n");
	printf(" FT2232C(MPSSE) TEST PROGRAM\r\n");
	printf(" Copyright (C)2005 Y.Sakamoto\r\n");
	printf(" (http://www.neko.ne.jp/~freewing/)\r\n");
	printf("============================================\r\n");

	if (argc != 2)
	{
		printf("Parameter Error\r\n");
		exit (1);
	}

	//==============================================
	//	FTDI OPEN
	//==============================================
	printf("FT2232C OPEN...");
	bFlag	= FTDI_OpenDevice();
	printf("%s\r\n", STR_OKNG(bFlag));
	if (!bFlag)	exit (2);

	//==============================================
	//	OPERATION
	//==============================================
	bFlag	= FtTest(atoi(argv[1]));
	printf("Operation...");
	printf("%s\r\n", STR_OKNG(bFlag));

	//==============================================
	//	FTDI CLOSE
	//==============================================
	FTDI_CloseDevice();

	if (!bFlag)	exit (3);
	//==============================================

	exit (0);
}


//==============================================
BOOL	FtTest(int nParam)
{
	BOOL	bFlag;

	//==============================================
	//	FTDI INIT
	//==============================================
	bFlag	= FTDI_SetMpsse();
	printf("FT2232C INIT...");
	printf("%s\n", STR_OKNG(bFlag));
	if (!bFlag)		return (FALSE);

	bFlag	= FTDI_SetParameter(3, 1000, 4096, 0, 0x08);
	printf("FT2232C PARAMETER...");
	printf("%s\n", STR_OKNG(bFlag));
	if (!bFlag)		return (FALSE);

	//==============================================
	//	OPERATION
	//==============================================
	printf("Operation...%d\n", nParam);
		switch (nParam)
		{
					//	Tx,Rx,Imm
		case	11:	//	 1, 0,  0
		case	12:	//	 0, 1,  0
		case	13:	//	 1, 1,  0
		case	16:	//	 0, 1,  1
		case	17:	//	 1, 1,  1
			bFlag	= FTDI_SetClkDivisor(1-1);		//	6MHz
			if (!bFlag)		return (FALSE);
			bFlag	= TdiBench(nParam % 10);
			break;

					//	Tx,Rx,Imm
		case	21:	//	 1, 0,  0
		case	22:	//	 0, 1,  0
		case	23:	//	 1, 1,  0
		case	26:	//	 0, 1,  1
		case	27:	//	 1, 1,  1
			bFlag	= FTDI_SetClkDivisor(6-1);		//	1MHz
			if (!bFlag)		return (FALSE);
			bFlag	= TdiBench(nParam % 10);
			break;

		case	30:
			bFlag	= TmsSend(1000);
			break;

		default:
			printf("Error\n");
			bFlag	= FALSE;
			break;
		}

	return (bFlag);
}


//==============================================
//	Transfer
DWORD	adwBenchLength1[]	=
	{
		0x00100,
		0x00200,
		0x00400,
		0x00800,
		0x01000,
		0x02000,
		0x04000,
		0x08000,
		0x0F7FE,
		0x10000,
		0
	};


//==============================================
//	TDI TEST
BOOL	TdiBench(int nParam)
{
	BOOL	bFlag;
	int		nLoopCnt;
	DWORD	dwLength;
	DWORD	*pBenchLength;
	int	i;

	printf( "//==============================================\n");
	printf( "Data Output    = %s\n", (nParam & 1)? "Enable": "Disable");
	printf( "Data Input     = %s\n", (nParam & 2)? "Enable": "Disable");
	printf( "Send Immediate = %s\n", (nParam & 4)? "Enable": "Disable");
	printf( "\n");

	nLoopCnt	= 1000;
	pBenchLength	= adwBenchLength1;
	i	= 0;
	while (pBenchLength[i])
	{
		dwLength	= pBenchLength[i];
		if ((nParam & 2) && (dwLength > 0x0F7FE))
		{
			bFlag	= TRUE;
			break;
		}
		else
		{
			bFlag	= TdiBenchSub(nLoopCnt, dwLength, 0x55, (nParam & 1), (nParam & 2), (nParam & 4));
		}
		if (!bFlag)	break;
		i++;
	}

	return (bFlag);
}


//==============================================
BOOL	TdiBenchSub(int nLoopCnt, DWORD dwLength, BYTE byTxData, BOOL bTxEna, BOOL bRxEna, BOOL bSendImmEna)
{
	BOOL	bFlag;
	BYTE	tx_buf[BUFFER_MAX];
	BYTE	rx_buf[BUFFER_MAX];
	DWORD	tx_cnt;

    clock_t startClock;
    clock_t endClock;
	double	dExecutionTime;

	printf( "Transfer Size  = 0x%06lX bytes (Data=0x%02X)\n", dwLength, byTxData);

	//	BenchMark MPSSE
	{
		BYTE	cmd;

		//	Make Write Buffer Data
		tx_cnt		= 0;

		//	TDI
		cmd	= 0x00;
		if (bTxEna)	cmd	|= 0x19;	// clk data bytes out on -ve clk LSB
		if (bRxEna)	cmd	|= 0x29;	// clk data bytes in on +ve clk LSB
		tx_buf[tx_cnt++]	= cmd;
		tx_buf[tx_cnt++]	= (BYTE)((dwLength-1) & 0xFF);	//	L,Byte Length
		tx_buf[tx_cnt++]	= (BYTE)((dwLength-1)>>8);		//	H
		if (bTxEna)
		{
			for(DWORD t=0; t<dwLength; t++)
			{
				tx_buf[tx_cnt++]	= byTxData;	//	Byte Data[LSB]
			}
		}

		if (bSendImmEna)	tx_buf[tx_cnt++]	= 0x87;	//	Send Immediate
	}

	{
		int j;

		//	Purge Buffer
		bFlag	= FTDI_PurgeBuffer();
		if (!bFlag)		return (FALSE);
		Sleep(20);	// wait for all the USB stuff to complete

		//	Transfer Loop
		startClock	= clock();
		for (j=0; j<nLoopCnt; j++)
		{
			bFlag	= FTDI_TransferBuffer(tx_buf, tx_cnt);
			if (!bFlag)
			{
				printf("FTDI_TransferBuffer NG\n");
				break;
			}

			if (bRxEna) {
				bFlag	= FTDI_WaitReceiveBuffer(rx_buf, dwLength, 1000);
				if (!bFlag)
				{
					printf("FTDI_WaitReceiveBuffer NG\n");
					break;
				}
			}
		}
		endClock	= clock();

		//	Display Transfer Rate
		dExecutionTime	= ((double)(endClock - startClock))/CLOCKS_PER_SEC;
		printf( "Execution Time	= %7.3f sec\n", dExecutionTime);
		printf( "Transfer rate  = 0x%06lX bytes/sec\n", (long)((dwLength*j)/dExecutionTime));
		printf( "\n");

		//	Sleep
		Sleep(2000);
	}

	return (bFlag);
}


//==============================================
//	TMS TEST
BOOL	TmsSend(int nLoopCnt)
{
	BOOL	bFlag;
	BYTE	tx_buf[BUFFER_MAX];
	DWORD	tx_cnt;
	int i;

	printf( "//==============================================\n");
	printf( "Tms Test\n");

	//==============================================
	//	Make Write Buffer Data
	tx_cnt		= 0;
	tx_buf[tx_cnt++]	= 0x80;	//	Set I/O Low Bits
	tx_buf[tx_cnt++]	= 0x00;
	tx_buf[tx_cnt++]	= 0x0B;

	tx_buf[tx_cnt++]	= 0x4B;	//	TMS
	tx_buf[tx_cnt++]	= 0x03;
	tx_buf[tx_cnt++]	= 0x97;

	tx_buf[tx_cnt++]	= 0x1B;	//	TDI
	tx_buf[tx_cnt++]	= 0x03;
	tx_buf[tx_cnt++]	= 0x14;

	tx_buf[tx_cnt++]	= 0x4B;	//	TMS
	tx_buf[tx_cnt++]	= 0x03;
	tx_buf[tx_cnt++]	= 0x0E;

	tx_buf[tx_cnt++]	= 0x1B;	//	TDI
	tx_buf[tx_cnt++]	= 0x03;
	tx_buf[tx_cnt++]	= 0x0B;

	//	Transfer Loop
	for (i=0; i<nLoopCnt; i++)
	{
		bFlag	= FTDI_TransferBuffer(tx_buf, tx_cnt);
		if (!bFlag)
		{
			printf("FTDI_TransferBuffer NG\n");
			break;
		}
	}

	printf( "\n");
	printf( "End\n");
	printf( "\n");

	return (bFlag);
}


